# Generated from QueryParser.g4 by ANTLR 4.13.2
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,17,36,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,1,0,1,0,1,0,5,
        0,14,8,0,10,0,12,0,17,9,0,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,3,1,27,
        8,1,1,2,1,2,1,2,1,3,1,3,1,4,1,4,1,4,0,0,5,0,2,4,6,8,0,2,1,0,5,8,
        1,0,9,17,34,0,10,1,0,0,0,2,26,1,0,0,0,4,28,1,0,0,0,6,31,1,0,0,0,
        8,33,1,0,0,0,10,15,3,2,1,0,11,12,5,3,0,0,12,14,3,2,1,0,13,11,1,0,
        0,0,14,17,1,0,0,0,15,13,1,0,0,0,15,16,1,0,0,0,16,18,1,0,0,0,17,15,
        1,0,0,0,18,19,5,0,0,1,19,1,1,0,0,0,20,27,3,4,2,0,21,27,3,8,4,0,22,
        23,5,4,0,0,23,27,3,4,2,0,24,25,5,4,0,0,25,27,3,8,4,0,26,20,1,0,0,
        0,26,21,1,0,0,0,26,22,1,0,0,0,26,24,1,0,0,0,27,3,1,0,0,0,28,29,5,
        2,0,0,29,30,3,6,3,0,30,5,1,0,0,0,31,32,7,0,0,0,32,7,1,0,0,0,33,34,
        7,1,0,0,34,9,1,0,0,0,2,15,26
    ]

class QueryParser ( Parser ):

    grammarFileName = "QueryParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "'bipartite'", "'complete'", 
                     "'connected'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "'planar'", "'tree'" ]

    symbolicNames = [ "<INVALID>", "WHITESPACE", "INTEGER", "SEPERATOR", 
                      "NOT", "VERTEX", "EDGE", "BLOCK", "COMPONENT", "ACYCLIC", 
                      "BIPARTITE", "COMPLETE", "CONNECTED", "EULERIAN", 
                      "HAMILTONIAN", "NO_ISOLATED_V", "PLANAR", "TREE" ]

    RULE_query = 0
    RULE_expr = 1
    RULE_numEntityProperty = 2
    RULE_entity = 3
    RULE_boolProperty = 4

    ruleNames =  [ "query", "expr", "numEntityProperty", "entity", "boolProperty" ]

    EOF = Token.EOF
    WHITESPACE=1
    INTEGER=2
    SEPERATOR=3
    NOT=4
    VERTEX=5
    EDGE=6
    BLOCK=7
    COMPONENT=8
    ACYCLIC=9
    BIPARTITE=10
    COMPLETE=11
    CONNECTED=12
    EULERIAN=13
    HAMILTONIAN=14
    NO_ISOLATED_V=15
    PLANAR=16
    TREE=17

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.2")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class QueryContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expr(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(QueryParser.ExprContext)
            else:
                return self.getTypedRuleContext(QueryParser.ExprContext,i)


        def EOF(self):
            return self.getToken(QueryParser.EOF, 0)

        def SEPERATOR(self, i:int=None):
            if i is None:
                return self.getTokens(QueryParser.SEPERATOR)
            else:
                return self.getToken(QueryParser.SEPERATOR, i)

        def getRuleIndex(self):
            return QueryParser.RULE_query

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitQuery" ):
                return visitor.visitQuery(self)
            else:
                return visitor.visitChildren(self)




    def query(self):

        localctx = QueryParser.QueryContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_query)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 10
            self.expr()
            self.state = 15
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==3:
                self.state = 11
                self.match(QueryParser.SEPERATOR)
                self.state = 12
                self.expr()
                self.state = 17
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 18
            self.match(QueryParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ExprContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return QueryParser.RULE_expr

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class SimpleExprContext(ExprContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a QueryParser.ExprContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def numEntityProperty(self):
            return self.getTypedRuleContext(QueryParser.NumEntityPropertyContext,0)

        def boolProperty(self):
            return self.getTypedRuleContext(QueryParser.BoolPropertyContext,0)


        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSimpleExpr" ):
                return visitor.visitSimpleExpr(self)
            else:
                return visitor.visitChildren(self)


    class NotExprContext(ExprContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a QueryParser.ExprContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NOT(self):
            return self.getToken(QueryParser.NOT, 0)
        def numEntityProperty(self):
            return self.getTypedRuleContext(QueryParser.NumEntityPropertyContext,0)

        def boolProperty(self):
            return self.getTypedRuleContext(QueryParser.BoolPropertyContext,0)


        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNotExpr" ):
                return visitor.visitNotExpr(self)
            else:
                return visitor.visitChildren(self)



    def expr(self):

        localctx = QueryParser.ExprContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_expr)
        try:
            self.state = 26
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,1,self._ctx)
            if la_ == 1:
                localctx = QueryParser.SimpleExprContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 20
                self.numEntityProperty()
                pass

            elif la_ == 2:
                localctx = QueryParser.SimpleExprContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 21
                self.boolProperty()
                pass

            elif la_ == 3:
                localctx = QueryParser.NotExprContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 22
                self.match(QueryParser.NOT)
                self.state = 23
                self.numEntityProperty()
                pass

            elif la_ == 4:
                localctx = QueryParser.NotExprContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 24
                self.match(QueryParser.NOT)
                self.state = 25
                self.boolProperty()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NumEntityPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def INTEGER(self):
            return self.getToken(QueryParser.INTEGER, 0)

        def entity(self):
            return self.getTypedRuleContext(QueryParser.EntityContext,0)


        def getRuleIndex(self):
            return QueryParser.RULE_numEntityProperty

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNumEntityProperty" ):
                return visitor.visitNumEntityProperty(self)
            else:
                return visitor.visitChildren(self)




    def numEntityProperty(self):

        localctx = QueryParser.NumEntityPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_numEntityProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 28
            self.match(QueryParser.INTEGER)
            self.state = 29
            self.entity()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def VERTEX(self):
            return self.getToken(QueryParser.VERTEX, 0)

        def EDGE(self):
            return self.getToken(QueryParser.EDGE, 0)

        def BLOCK(self):
            return self.getToken(QueryParser.BLOCK, 0)

        def COMPONENT(self):
            return self.getToken(QueryParser.COMPONENT, 0)

        def getRuleIndex(self):
            return QueryParser.RULE_entity

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntity" ):
                return visitor.visitEntity(self)
            else:
                return visitor.visitChildren(self)




    def entity(self):

        localctx = QueryParser.EntityContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_entity)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 31
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 480) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BoolPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ACYCLIC(self):
            return self.getToken(QueryParser.ACYCLIC, 0)

        def BIPARTITE(self):
            return self.getToken(QueryParser.BIPARTITE, 0)

        def COMPLETE(self):
            return self.getToken(QueryParser.COMPLETE, 0)

        def CONNECTED(self):
            return self.getToken(QueryParser.CONNECTED, 0)

        def EULERIAN(self):
            return self.getToken(QueryParser.EULERIAN, 0)

        def HAMILTONIAN(self):
            return self.getToken(QueryParser.HAMILTONIAN, 0)

        def NO_ISOLATED_V(self):
            return self.getToken(QueryParser.NO_ISOLATED_V, 0)

        def PLANAR(self):
            return self.getToken(QueryParser.PLANAR, 0)

        def TREE(self):
            return self.getToken(QueryParser.TREE, 0)

        def getRuleIndex(self):
            return QueryParser.RULE_boolProperty

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBoolProperty" ):
                return visitor.visitBoolProperty(self)
            else:
                return visitor.visitChildren(self)




    def boolProperty(self):

        localctx = QueryParser.BoolPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_boolProperty)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 33
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 261632) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








# Generated from QueryParser.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .QueryParser import QueryParser
else:
    from QueryParser import QueryParser

# This class defines a complete generic visitor for a parse tree produced by QueryParser.

class QueryParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by QueryParser#query.
    def visitQuery(self, ctx:QueryParser.QueryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by QueryParser#simpleExpr.
    def visitSimpleExpr(self, ctx:QueryParser.SimpleExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by QueryParser#notExpr.
    def visitNotExpr(self, ctx:QueryParser.NotExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by QueryParser#numEntityProperty.
    def visitNumEntityProperty(self, ctx:QueryParser.NumEntityPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by QueryParser#entity.
    def visitEntity(self, ctx:QueryParser.EntityContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by QueryParser#boolProperty.
    def visitBoolProperty(self, ctx:QueryParser.BoolPropertyContext):
        return self.visitChildren(ctx)



del QueryParser


from graphreveal.translator import QueryParser, QueryParserVisitor

BOOL_PROPERTY_MAP = {
    QueryParser.ACYCLIC: "acyclic = TRUE",
    QueryParser.BIPARTITE: "bipartite = TRUE",
    QueryParser.COMPLETE: "vertices * (vertices - 1) / 2 = edges",
    QueryParser.CONNECTED: "components = 1",
    QueryParser.EULERIAN: "eulerian = TRUE",
    QueryParser.HAMILTONIAN: "hamiltonian = TRUE",
    QueryParser.NO_ISOLATED_V: "degree_min > 0",
    QueryParser.PLANAR: "planar = TRUE",
    QueryParser.TREE: "acyclic = TRUE AND components = 1",
}

NUM_ENTITY_PROPERTY_MAP = {
    QueryParser.VERTEX: "vertices",
    QueryParser.EDGE: "edges",
    QueryParser.BLOCK: "blocks",
    QueryParser.COMPONENT: "components",
}


class QueryTranslator(QueryParserVisitor):
    def visitQuery(self, ctx: QueryParser.QueryContext) -> str:
        return " AND ".join(
            [self.visit(ctx.expr(i)) for i in range(ctx.getChildCount() // 2)]
        )

    def visitSimpleExpr(self, ctx: QueryParser.SimpleExprContext):
        return self.visitChildren(ctx)

    def visitNotExpr(self, ctx: QueryParser.NotExprContext):
        return "NOT " + self.visitChildren(ctx)

    def visitNumEntityProperty(self, ctx: QueryParser.NumEntityPropertyContext):
        num = ctx.INTEGER().getText()
        entity = self.visit(ctx.entity())
        return NUM_ENTITY_PROPERTY_MAP[entity] + " = " + num

    def visitEntity(self, ctx: QueryParser.EntityContext):
        return ctx.getChild(0).symbol.type

    def visitBoolProperty(self, ctx: QueryParser.BoolPropertyContext):
        return BOOL_PROPERTY_MAP[ctx.getChild(0).symbol.type]



import pytest
from antlr4 import InputStream, CommonTokenStream

from graphreveal.translator import QueryLexer, QueryParser


@pytest.mark.parametrize(
    "valid_query",
    [
        "6 vertices",
        "18 edges",
        "2 components",
        "100 nodes",
        "1 vertex; 1 edge; 1 component; 1 node",
        "2 V, 0 E, 1 C",
        "not acyclic, ! bipartite, connected, eulerian",
        "hamiltonian, euler, hamilton",
        "5 V; !planar",
        "6 vertices, not complete",
        "not 2 vertices, 1 edge",
        "forest; tree; no isolated vertices",
        "2 biconnected components, 2 blocks",
    ],
)
def test_valid_query(valid_query):
    lexer = QueryLexer(InputStream(valid_query))
    parser = QueryParser(CommonTokenStream(lexer))

    parser.query()
    assert parser.getNumberOfSyntaxErrors() == 0


@pytest.mark.parametrize(
    "invalid_query",
    [
        "",
        "1",
        ",",
        "wrong",
        "vertices",
        "edges",
        "component",
        "1 bipartite",
        "bipartite 1",
        "6 vertices ,, 8 edges",
        "6 V 8 E",
        "isolated vertices",
        "biconnected component component",
    ],
)
def test_invalid_query(invalid_query):
    lexer = QueryLexer(InputStream(invalid_query))
    parser = QueryParser(CommonTokenStream(lexer))

    parser.query()
    assert parser.getNumberOfSyntaxErrors() > 0



from .generated.QueryLexer import QueryLexer as QueryLexer
from .generated.QueryParser import QueryParser as QueryParser
from .generated.QueryParserVisitor import QueryParserVisitor as QueryParserVisitor
from .QueryTranslator import QueryTranslator as QueryTranslator
from .translate import translate as translate



from typer.testing import CliRunner
from graphreveal.cli import app

runner = CliRunner()


def test_app():
    result = runner.invoke(app, ["search", "1 vertex"])
    assert result.exit_code == 0
    assert result.output == "@\n"


def test_app_with_count():
    result = runner.invoke(app, ["search", "6 vertices, connected", "--count"])
    assert result.exit_code == 0
    assert result.stdout == "112\n"


def test_app_with_sql():
    result = runner.invoke(app, ["to-sql", "5 vertices, planar"])
    assert result.exit_code == 0
    assert (
        result.stdout == "SELECT * FROM graphs WHERE vertices = 5 AND planar = TRUE\n"
    )



import os
import sqlite3

from platformdirs import user_data_dir

DATABASE_PATH = os.path.join(
    user_data_dir(appname="graphreveal", appauthor="graphreveal"), "graphs.db"
)


class ParsingError(Exception):
    def __init__(self, message: str, errors_coordinates: list[tuple[int, int, int]]):
        # todo: show all errors, not just the first one
        error_coordinates = errors_coordinates[0]
        self.message = message
        self.error_line, self.error_column, self.error_length = error_coordinates


def get_ids(sql_query: str) -> list[str]:
    sql_query = sql_query.replace("*", "id", 1)
    con = sqlite3.connect(DATABASE_PATH)
    cur = con.cursor()

    result = cur.execute(sql_query).fetchall()
    return [graph_id for (graph_id,) in result]



import pytest
from graphreveal import get_ids
from graphreveal.translator import translate


@pytest.mark.parametrize(
    "query,expected_count",
    [
        ("1 vertex", 1),
        ("2 vertices", 2),
        ("6 vertices, connected", 112),
        ("6 vertices, connected, not 15 edges", 111),
        ("6 vertices, connected, not complete", 111),
        ("1 edge, connected", 1),
        ("3 edges, connected", 3),
        ("6 edges, connected", 30),
        ("!bipartite, acyclic", 0),
        ("5 vertices, complete", 1),
        ("7 vertices, eulerian", 37),
        ("5 vertices, connected, planar", 20),
        ("5 vertices, tree", 3),
        ("5 vertices, 2 components", 8),
        ("3 edges, no isolated vertices", 5),
        ("1 vertex, hamiltonian", 1),
        ("2 vertices, hamiltonian", 0),
        ("3 vertices, hamiltonian", 1),
        ("4 vertices, hamiltonian", 3),
        ("5 vertices, hamiltonian", 8),
        ("6 vertices, hamiltonian", 48),
        ("7 vertices, hamiltonian", 383),
        ("5 vertices, connected, 2 blocks", 5),
        ("5 vertices, connected, 1 block", 10),
    ],
)
def test_query_results(query, expected_count):
    sql_query = translate(query)
    assert len(get_ids(sql_query)) == expected_count



from antlr4 import InputStream, CommonTokenStream
from antlr4.error.ErrorListener import ErrorListener

from graphreveal import ParsingError
from graphreveal.translator import QueryLexer, QueryParser, QueryTranslator


class QueryErrorListener(ErrorListener):
    def __init__(self):
        self.errors = []

    def syntaxError(self, recognizer, offendingSymbol, line, column, msg, e):
        if offendingSymbol:
            length = offendingSymbol.stop - offendingSymbol.start + 1
        elif isinstance(recognizer, QueryLexer):
            length = len(msg.split("recognition error at: ")[1]) - 2
        else:  # this should not happen
            raise AssertionError("length of this error cannot be determined")
        length = max(1, length)
        self.errors.append((line, column, length))


def translate(input_text: str, print_parse_tree: bool = False) -> str:
    """Translates natural language query to SQL beginning with `SELECT * `"""
    lexer = QueryLexer(InputStream(input_text))
    parser = QueryParser(CommonTokenStream(lexer))

    lexer.removeErrorListeners()
    lexer_error_listener = QueryErrorListener()
    lexer.addErrorListener(lexer_error_listener)

    parser.removeErrorListeners()
    parser_error_listener = QueryErrorListener()
    parser.addErrorListener(parser_error_listener)

    tree = parser.query()
    if print_parse_tree:
        print(tree.toStringTree(recog=parser))

    if lexer_error_listener.errors:
        raise ParsingError("Your query is invalid", lexer_error_listener.errors)

    if parser_error_listener.errors:
        raise ParsingError("Your query is invalid", parser_error_listener.errors)

    translator = QueryTranslator()
    return "SELECT * FROM graphs WHERE " + translator.visit(tree)



import networkx as nx


def _find_closure(G: nx.Graph) -> nx.Graph:
    """Returns a graph cl(G) defined as in Bondy-Chvátal Theorem."""
    G = G.copy()
    n = G.number_of_nodes()
    edges_to_add = []
    for v in G.nodes:
        for u in G.nodes - G.neighbors(v) - {v}:
            if G.degree[v] + G.degree[u] >= n:
                edges_to_add.append((v, u))

    while edges_to_add:
        v, u = edges_to_add.pop()
        G.add_edge(v, u)
        for w in G.nodes - G.neighbors(v) - {v}:
            if G.degree[w] + G.degree[v] >= n:
                edges_to_add.append((w, v))
        for w in G.nodes - G.neighbors(u) - {u}:
            if G.degree[w] + G.degree[u] >= n:
                edges_to_add.append((w, u))

    return G


def is_hamiltonian(G: nx.Graph) -> bool:
    """Checks whether a graph G is hamiltonian or not."""
    n = G.number_of_nodes()
    if n == 1:
        return True
    if n == 2:
        return False
    if not nx.is_connected(G) or nx.is_tree(G):
        return False
    if not nx.is_biconnected(G):
        return False

    cl_G = _find_closure(G)
    if (
        cl_G.number_of_edges()
        == cl_G.number_of_nodes() * (cl_G.number_of_nodes() - 1) / 2
    ):
        return True

    return nx.isomorphism.GraphMatcher(G, nx.cycle_graph(n)).subgraph_is_monomorphic()


def max_degree(graph: nx.Graph) -> int:
    return max(d for _, d in graph.degree())


def min_degree(graph: nx.Graph) -> int:
    return min(d for _, d in graph.degree())



import os
from sqlite3 import OperationalError

import rich
import typer
from rich.prompt import Confirm

from graphreveal import get_ids, ParsingError, DATABASE_PATH
from graphreveal.db_creator import create_db
from graphreveal.translator import translate

app = typer.Typer(no_args_is_help=True)


def _print_parsing_error(query: str, e: ParsingError):
    rich.print("[bold red]Query error:\n")
    for i, query_line in enumerate(query.split("\n"), start=1):
        query_line += " "
        if i == e.error_line:
            rich.print(
                f"  [not bold cyan]{query_line[: e.error_column]}"
                f"[bold red]{query_line[e.error_column : e.error_column + e.error_length]}"
                f"[not bold cyan]{query_line[e.error_column + e.error_length :]}"
            )
            rich.print(f"  {' ' * e.error_column}[red]{'^' * e.error_length}")
        else:
            rich.print(f"  [not bold cyan]{query_line}\n")
    rich.print()


@app.command()
def search(query: str, count: bool = False):
    """
    Get all graphs with given properties.
    """
    try:
        sql_query = translate(query)
        if count:
            print(len(get_ids(sql_query)))
        else:
            print("\n".join(get_ids(sql_query)))
    except ParsingError as e:
        _print_parsing_error(query, e)
    except OperationalError as e:
        rich.print("[bold red]Error:", str(e) + ".")
        rich.print(
            "Try to create the database first. Run `[cyan]graphreveal create-database[/cyan]`."
        )


@app.command()
def create_database(n: int = 7):
    """
    Create the database.
    """
    if n > 9 or n < 1:
        rich.print("[bold red]Error: Choose n between 1 and 9.")
        return

    try:
        if not os.path.exists(DATABASE_PATH) or Confirm.ask(
            "Are you sure you want to overwrite the database?"
        ):
            create_db(n)
    except OperationalError as e:
        rich.print("[bold red]Error:", str(e) + ".")


@app.command()
def to_sql(query: str):
    """
    Translate your query to SQL.
    """
    try:
        sql_query = translate(query)
        print(sql_query)
    except ParsingError as e:
        _print_parsing_error(query, e)


if __name__ == "__main__":
    app()



import importlib.resources
import os
import sqlite3

import networkx as nx
from rich.progress import track

from graphreveal import DATABASE_PATH
from . import util


def create_db(max_n):
    os.makedirs(os.path.dirname(DATABASE_PATH), exist_ok=True)
    con = sqlite3.connect(DATABASE_PATH)
    cur = con.cursor()

    cur.execute("DROP TABLE IF EXISTS graphs")

    cur.execute(
        """ CREATE TABLE graphs (
                id VARCHAR(20) PRIMARY KEY,
                vertices INT NOT NULL,
                edges INT NOT NULL,
                acyclic BOOLEAN NOT NULL,
                bipartite BOOLEAN NOT NULL,
                eulerian BOOLEAN NOT NULL,
                hamiltonian BOOLEAN NOT NULL,
                planar BOOLEAN NOT NULL,
                blocks INT NOT NULL,
                components INT NOT NULL,
                degree_max INT NOT NULL,
                degree_min INT NOT NULL
            )"""
    )

    all_graphs = []

    for n in range(1, max_n + 1):
        with (
            importlib.resources.files("graphreveal")
            / "db_creator"
            / "data"
            / f"graph{n}.g6"
        ).open(encoding="utf-8") as f:
            all_graphs += f.read().strip().split("\n")

    for graph_g6 in track(all_graphs, description="Creating the database"):
        graph = nx.from_graph6_bytes(str.encode(graph_g6))
        cur.execute(
            "INSERT INTO graphs VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
            [
                graph_g6,
                graph.number_of_nodes(),
                graph.number_of_edges(),
                nx.is_forest(graph),
                nx.is_bipartite(graph),
                nx.is_eulerian(graph),
                util.is_hamiltonian(graph),
                nx.is_planar(graph),
                len(list(nx.biconnected_components(graph))),
                nx.number_connected_components(graph),
                util.max_degree(graph),
                util.min_degree(graph),
            ],
        )

    con.commit()
    con.close()
